@@ -1,50 +1,51 @@
-/*
- *
- * Copyright 2015 gRPC authors.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
- *
- */
+//
+//
+// Copyright 2015 gRPC authors.
+//
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+//
+//     http://www.apache.org/licenses/LICENSE-2.0
+//
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+//
+//
 
-/* Posix implementation for gpr threads. */
+// Posix implementation for gpr threads.
 
 #include <grpc/support/port_platform.h>
 
+#include <grpc/support/time.h>
+
 #ifdef GPR_POSIX_SYNC
 
-#include "src/core/lib/gprpp/thd.h"
-
-#include <grpc/support/alloc.h>
-#include <grpc/support/log.h>
-#include <grpc/support/sync.h>
-#include <grpc/support/thd_id.h>
 #include <pthread.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
+#include <grpc/support/log.h>
+#include <grpc/support/sync.h>
+#include <grpc/support/thd_id.h>
+
 #include "src/core/lib/gpr/useful.h"
 #include "src/core/lib/gprpp/fork.h"
-#include "src/core/lib/gprpp/memory.h"
+#include "src/core/lib/gprpp/thd.h"
 
 namespace grpc_core {
 namespace {
 class ThreadInternalsPosix;
+
 struct thd_arg {
   ThreadInternalsPosix* thread;
-  void (*body)(void* arg); /* body of a thread */
-  void* arg;               /* argument to a thread */
-  const char* name;        /* name of thread. Can be nullptr. */
+  void (*body)(void* arg);  // body of a thread
+  void* arg;                // argument to a thread
+  const char* name;         // name of thread. Can be nullptr.
   bool joinable;
   bool tracked;
 };
@@ -77,8 +78,8 @@ class ThreadInternalsPosix : public internal::ThreadInternalsInterface {
     gpr_mu_init(&mu_);
     gpr_cv_init(&ready_);
     pthread_attr_t attr;
-    /* don't use gpr_malloc as we may cause an infinite recursion with
-     * the profiling code */
+    // don't use gpr_malloc as we may cause an infinite recursion with
+    // the profiling code
     thd_arg* info = static_cast<thd_arg*>(malloc(sizeof(*info)));
     GPR_ASSERT(info != nullptr);
     info->thread = this;
@@ -112,12 +113,12 @@ class ThreadInternalsPosix : public internal::ThreadInternalsInterface {
                       free(v);
                       if (arg.name != nullptr) {
 #if GPR_APPLE_PTHREAD_NAME
-                        /* Apple supports 64 characters, and will
-                         * truncate if it's longer. */
+                        // Apple supports 64 characters, and will
+                        // truncate if it's longer.
                         pthread_setname_np(arg.name);
 #elif GPR_LINUX_PTHREAD_NAME
-                        /* Linux supports 16 characters max, and will
-                         * error if it's longer. */
+                        // Linux supports 16 characters max, and will
+                        // error if it's longer.
                         char buf[16];
                         size_t buf_len = GPR_ARRAY_SIZE(buf) - 1;
                         strncpy(buf, arg.name, buf_len);
@@ -148,7 +149,7 @@ class ThreadInternalsPosix : public internal::ThreadInternalsInterface {
     GPR_ASSERT(pthread_attr_destroy(&attr) == 0);
 
     if (!(*success)) {
-      /* don't use gpr_free, as this was allocated using malloc (see above) */
+      // don't use gpr_free, as this was allocated using malloc (see above)
       free(info);
       if (options.tracked()) {
         Fork::DecThreadCount();
@@ -199,6 +200,11 @@ Thread::Thread(const char* thd_name, void (*thd_body)(void* arg), void* arg,
 }  // namespace grpc_core
 
 // The following is in the external namespace as it is exposed as C89 API
-gpr_thd_id gpr_thd_currentid(void) { return (gpr_thd_id)pthread_self(); }
+gpr_thd_id gpr_thd_currentid(void) {
+  // Use C-style casting because Linux and OSX have different definitions
+  // of pthread_t so that a single C++ cast doesn't handle it.
+  // NOLINTNEXTLINE(google-readability-casting)
+  return (gpr_thd_id)pthread_self();
+}
 
-#endif /* GPR_POSIX_SYNC */
+#endif  // GPR_POSIX_SYNC
