@@ -14,8 +14,8 @@
 // limitations under the License.
 //
 
-#ifndef GRPC_CORE_LIB_LOAD_BALANCING_LB_POLICY_H
-#define GRPC_CORE_LIB_LOAD_BALANCING_LB_POLICY_H
+#ifndef GRPC_SRC_CORE_LIB_LOAD_BALANCING_LB_POLICY_H
+#define GRPC_SRC_CORE_LIB_LOAD_BALANCING_LB_POLICY_H
 
 #include <grpc/support/port_platform.h>
 
@@ -40,6 +40,7 @@
 #include "src/core/lib/channel/channel_args.h"
 #include "src/core/lib/debug/trace.h"
 #include "src/core/lib/gprpp/debug_location.h"
+#include "src/core/lib/gprpp/dual_ref_counted.h"
 #include "src/core/lib/gprpp/orphanable.h"
 #include "src/core/lib/gprpp/ref_counted.h"
 #include "src/core/lib/gprpp/ref_counted_ptr.h"
@@ -176,6 +177,7 @@ class LoadBalancingPolicy : public InternallyRefCounted<LoadBalancingPolicy> {
     /// implementation does not take ownership, so any data that needs to be
     /// used after returning must be copied.
     struct FinishArgs {
+      absl::string_view peer_address;
       absl::Status status;
       MetadataInterface* trailing_metadata;
       BackendMetricAccessor* backend_metric_accessor;
@@ -256,11 +258,13 @@ class LoadBalancingPolicy : public InternallyRefCounted<LoadBalancingPolicy> {
   /// Currently, pickers are always accessed from within the
   /// client_channel data plane mutex, so they do not have to be
   /// thread-safe.
-  class SubchannelPicker : public RefCounted<SubchannelPicker> {
+  class SubchannelPicker : public DualRefCounted<SubchannelPicker> {
    public:
-    SubchannelPicker() = default;
+    SubchannelPicker();
 
     virtual PickResult Pick(PickArgs args) = 0;
+
+    void Orphan() override {}
   };
 
   /// A proxy object implemented by the client channel and used by the
@@ -435,4 +439,4 @@ class LoadBalancingPolicy : public InternallyRefCounted<LoadBalancingPolicy> {
 
 }  // namespace grpc_core
 
-#endif  // GRPC_CORE_LIB_LOAD_BALANCING_LB_POLICY_H
+#endif  // GRPC_SRC_CORE_LIB_LOAD_BALANCING_LB_POLICY_H
